(function (global) {
    "use strict";

    global.ND = global.ND || {};
    global.ND.Portal = global.ND.Portal || {};
    global.ND.Portal.Tables = (function() {

		var handlePageToggle = (selectedPage, currentPage, tableName, createRowFunction, serviceEndpointName, recordId, tableConfiiguration, postBackMethod) => {
			const urlParams = new URLSearchParams(window.location.search);
			let id = "";
			if (tableName.includes("lessees")) {
				id = recordId;
			} else if (createRowFunction.name === "createAttachmentItem") {
                id = $("#nd-view-message-modal").attr("data-related-id");
            } else {
				id = urlParams.get('id');
			}
			var requestUrl = `/Services?name=${serviceEndpointName}&page={0}&id=${id}`;
			currentPage = ND.Portal.Tables.handleTablePaging(currentPage,
				selectedPage,
				requestUrl,
				tableName,
				createRowFunction,
				handlePageToggle, tableConfiiguration, postBackMethod);
        }

        var handleSubgridPageToggle = (selectedPage, currentPage, tableName, createRowFunction, setOnClickEvent, tableConfiguration, handleSubgridPostback) => {
			currentPage = ND.Portal.Tables.handleSubgridTablePaging(selectedPage,
				tableName,
				createRowFunction,
				setOnClickEvent,
                tableConfiguration, 
                handleSubgridPostback);
        }

        var handleSubgridTablePaging = (currentPage, selectorPrefix, createRowCallback, setOnClickEvent, pageConfiguration, subgridPostback) => {
            var orderOn, orderBy;
            var loadingIconSelector = `#${selectorPrefix}Loading`;
            $(loadingIconSelector).show();
            var noResultsMessage = `#${selectorPrefix}Table .empty-message`;
            $(noResultsMessage).hide();
            $(`#${selectorPrefix}Next`).addClass("disabled");
            $(`#${selectorPrefix}Previous`).addClass("disabled");
            
            var keyName;
            let rootIndex;
            let data = ND.Portal.Utilities.getPortalDataProperty('intersects', []);
            if (selectorPrefix === "wells") {
                keyName = "ndland_applicationwell";
                const tableSettings = pageConfiguration[selectorPrefix];
                orderOn = tableSettings?.OrderingConfiguration?.OrderOn;
                if (orderOn) {
                    orderBy = tableSettings?.OrderingConfiguration?.OrderBy || "ascending";
                }
            } else if (selectorPrefix === "products") {
                keyName = "ndland_applicationweedpest";
            } else if (selectorPrefix === "assigneeIntersects" || selectorPrefix === "lesseeIntersects") {
                keyName = "ndland_applicationassignee";
            } else if (selectorPrefix === "tracts") {
                keyName = "ndland_applicationtract";
            } else if (selectorPrefix === "leaseAgreements") {
                keyName = "ndland_applicationleaseagreement";
                
            } else if (selectorPrefix === "leaseAgreementOwnerships") {
                keyName = "ndland_applicationleaseagreementownership";
            } else {
                keyName = "ndland_applicationlandunit";
            }
            if (!subgridPostback || subgridPostback.name === "tractAcresTotalRow") {
                rootIndex = data.findIndex(x => x.intersectLogicalName == `${keyName}count`);
                if (rootIndex === -1) {
                    rootIndex = data.findIndex(x => x.intersectLogicalName == `${keyName}`);
                }
            } else {
                rootIndex = data.findIndex(x => x.intersectLogicalName == `${keyName}`);
                if (rootIndex === -1) {
                    rootIndex = data.findIndex(x => x.intersectLogicalName == `${keyName}count`);
                }       
            }

            if (rootIndex != -1) {
                data = data[rootIndex];
            }
            if (orderOn && data?.records) {
                data.records = _sortByKey(data.records, orderOn, orderBy);
            } else if (data?.records && selectorPrefix === "leaseAgreementOwnerships") {
                data.records = _sort(data.records, "name", "Assignee");
            }
            var itemsPerPage = 10;
            data = {
                itemsPerPageCount: itemsPerPage,
                totalRecordCount: data?.records?.length ?? 0,
                results: data?.records ?? []
            };
            if (data.results.length > 0) {
                $(`#${selectorPrefix}Table`).removeClass("table-empty");
                $(loadingIconSelector).hide();
                var dataPage = _formatSubgridRecords(data, currentPage, data.itemsPerPageCount);
                _refreshTable(dataPage, selectorPrefix, createRowCallback, true, subgridPostback);     
                _refreshPageList(currentPage, data.itemsPerPageCount, data, setOnClickEvent, selectorPrefix);
                if (pageConfiguration) {
                    resetTableSorting(selectorPrefix, pageConfiguration[selectorPrefix]?.OrderingConfiguration);
                }
            } else {
                $(loadingIconSelector).hide();
                _displayNoResults(selectorPrefix);
                var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);
                pageButtonList.empty();
                $(`#${selectorPrefix}Next`).removeClass("disabled");
                $(`#${selectorPrefix}Previous`).removeClass("disabled");
            }


            updateSubgridCount(selectorPrefix, data.results, data.totalRecordCount);
            
            return currentPage;
        };

        var _sortByKey = (array, key, order = 'ascending') => {
            return array.sort(function(a, b) {
                if (a[key] < b[key]) return (order === 'ascending') ? -1 : 1;
                if (a[key] > b[key]) return (order === 'descending') ? 1 : -1;
                return 0;
            });
        }

        var _sort = (array, key1, key2) => {
            return array.sort(function(a, b) {
                if (a[key1] < b[key1]) return -1;
                if (a[key1] > b[key1]) return 1;
                if (a[key1] === b[key1]) {
                    if (a[key2] < b[key2]) return -1;
                    if (a[key2] > b[key2]) return 1;
                }
                return 0;
            });
        }

        var _formatSubgridRecords = (data, currentPage, itemsPerPage = 25) => {
            const groupedPages = [];

            for (let i = 0; i < data.results.length; i += itemsPerPage) {
                groupedPages.push(data.results.slice(i, i + itemsPerPage));
            }

            var groupedPageIndex = (currentPage && currentPage !== 0) ? currentPage - 1 : 0;
            groupedPageIndex = Math.min(groupedPageIndex, groupedPages.length - 1);
            var dataPage = groupedPages[groupedPageIndex];
            dataPage = dataPage.map(item => ({
                Id: item.id,
                LandUnitName: item.name,
                LandUnitId: item.id,
                bonusPerAcre: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_nominatorcoalbidperacre")?.value,
                annualRent: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_nominatorcoaldelayrentalacreperyear")?.value,
                royalty: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_nominatorcoalroyaltyperton")?.value,
                lease: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_nominatorcoalterm")?.value,
                ndland_nominatorogbidperacre: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_nominatorogbidperacre")?.value,
                APINumber: item.APINumber,
                WellName: item.name,
                WellId: item.id,
                NDICFileNumber: item.NDICFileNumber,
                LeaseAgreementName: item.name,
                LeaseAgreementId: item.LeaseAgreementId ?? item.id,
                ExpirationDate: item.ExpirationDate,
                PrimaryLessee: item.PrimaryLessee,
                AnnualRent: item.AnnualRent,
                LeasedAcres: item.LeasedAcres,
                LeaseAgreementOwnershipId: item.id,
                Assignee: item.Assignee,
                Stratigraphic: item.Stratigraphic,
                PercentOwnership: item.PercentOwnership,
                PercentAssigned: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_workinginterestassigned")?.value * 100,
                AssigneeId: item.id,
                AssigneeName: item.name,
                TractName: item.name,
                TractId: item.id,
                LeaseNum: item.LeaseNum,
                EstAcres: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_estimatedacres")?.value ?? "",
                ProductName: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_weedpestcontroltypetext")?.value ?? "",
                ProductId: item.id,
                Cost: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_controlcost")?.value ?? "",
                Quantity: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_controlunitamount")?.value ?? "",
                Unit: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_controlunittype")?.value ?? "",
                Rate: (item.additionalValues || []).find(x => x.fieldLogicalName === "ndland_costperacre")?.value ?? ""
            }));
            return dataPage;
        }

        var writeAdditionalValuesToPortalData = (intersectRecord, tableName, fieldName, intersectValue, intersectType) => {
            if (!intersectRecord.additionalValues) {
                intersectRecord.additionalValues = [];
                intersectRecord.additionalValues.push({
                    "value" : intersectValue, 
                    "entity" : tableName, 
                    "fieldLogicalName" : fieldName, 
                    "type" : intersectType
                });
            } else {
                var index = intersectRecord.additionalValues.findIndex(x => x.fieldLogicalName === fieldName);
                if (index !== -1) {
                    intersectRecord.additionalValues[index] = {
                        "value": intersectValue,
                        "entity": tableName,
                        "fieldLogicalName": fieldName,
                        "type": intersectType
                    };
                } else {
                    intersectRecord.additionalValues.push({
                        "value": intersectValue,
                        "entity": tableName,
                        "fieldLogicalName": fieldName,
                        "type": intersectType
                    });
                }
            }
        }

		var handleTablePaging = (previousPage, currentPage, serviceUrl, selectorPrefix, createRowCallback, setOnClickEvent, pageConfiguration, postbackMethod) => {
            
            var loadingIconSelector = `#${selectorPrefix}Loading`;
            $(loadingIconSelector).show();
            $('#submit-btn').prop('disabled', true);
            $('[id$="reset-btn"]').prop('disabled', true);
            $('[id$="search-btn"]').prop('disabled', true);

            var noResultsMessage = `#${selectorPrefix}Table .empty-message`;
            $(noResultsMessage).hide();
            $(`#${selectorPrefix}Table`).removeClass('table-empty');
            $(`#${selectorPrefix}Next`).addClass("disabled");
            $(`#${selectorPrefix}Previous`).addClass("disabled");
            
            ND.Portal.API.appAjax('Retrieving records...', {
                type: "GET",
                url: serviceUrl.replace("{0}", currentPage),
                contentType: "application/json",
                success: function (response) {
                    var data = JSON.parse(response);
                    if (data.results.length > 0) {
                        $(loadingIconSelector).hide();
                        $('#submit-btn').prop('disabled', false);
                        $('[id$="reset-btn"]').prop('disabled', false);
                        $('[id$="search-btn"]').prop('disabled', false);

                        if (selectorPrefix === "allRelatedTracts") {
                            postbackMethod(data.results);
                            return;
                        }

                        _refreshTable(data.results, selectorPrefix, createRowCallback, false, postbackMethod);
                        _refreshPageList(currentPage, data.itemsPerPageCount, data, setOnClickEvent, selectorPrefix);
                    } else {
                        $(loadingIconSelector).hide();
                        $('#submit-btn').prop('disabled', false);
                        $('[id$="reset-btn"]').prop('disabled', false);
                        $('[id$="search-btn"]').prop('disabled', false);

                        _displayNoResults(selectorPrefix);
                        var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);
                        pageButtonList.empty();
                        currentPage = previousPage;
			            $(`#${selectorPrefix}Next`).removeClass("disabled");
            		    $(`#${selectorPrefix}Previous`).removeClass("disabled");
                        if (createRowCallback.name === "_createDocumentsRow") {
                            $("#documentsTable").closest(".table-container").hide();
                        }
                    }
                    if (createRowCallback.name === "createAttachmentItem") {
                        _generateMessageCenterDocuments(data, createRowCallback);
                    }
                    if (pageConfiguration) {
                        resetTableSorting(selectorPrefix, pageConfiguration[selectorPrefix]?.OrderingConfiguration);
                    }
                    updateSubgridCount(selectorPrefix, data.results, data.totalRecordCount);
                },
                failure: function (res) {
                    currentPage = previousPage;
                }
            });
            
            return currentPage;
        };

        var _generateMessageCenterDocuments = (data, createRowCallback) => {
            $("#attachments").children().remove();
            data.results.forEach(result => {
                var html = createRowCallback({ 
                    "FileName": result.DocumentTitle, 
                    "DocumentTitle": result.DocumentTitle, 
                    "DocumentId": result.DocumentId, 
                    "Subject": result.SubjectTitle, 
                    "Type": result.SurfaceDocumentType,
                    "DateUploaded": result.Date,
                    "StatusReason": result.StatusReason
                 });
                 $('#attachments').append(html);
                //  if (data.results.length === 1) {
                //     var tempDiv = document.createElement('div');
                //     tempDiv.innerHTML = html.trim();
                //     var content = tempDiv.firstChild.innerHTML;
                //     $("#attachments-list-single").show();
                //     $("#attachments-list-single").append(`<br>` + content);
                //     $("#attachments-list-multiple").hide();
                //  } else if (data.results.length > 1) {
                //     $('#attachments-list-multiple').show();
                //     $("#attachments-list-single").hide();
                //  }
            });

            if (data.results.length === 1) {
                $("#documentsTable").closest(".table-container").show();
               $("#attachments-list-single").show();
               $("#attachments-list-multiple").hide();
            } else if (data.results.length > 1) {
                $("#documentsTable").closest(".table-container").show();
               $('#attachments-list-multiple').show();
               $("#attachments-list-single").hide();
            } else {
               $("#documentsTable").closest(".table-container").hide();
               $("#attachments-list-single").hide();
               $("#attachments-list-multiple").hide();
            }
        }

        var handleApiTable = (itemsPerPage, previousPage, currentPage, serviceUrl, selectorPrefix, createRowCallback, setOnClickEvent, pageConfiguration) => {
      
            var loadingIconSelector = `#${selectorPrefix}Loading`;
            $(loadingIconSelector).show();
            $('#submit-btn').prop('disabled', true);
            $('#reset-btn').prop('disabled', true);

            var noResultsMessage = `#${selectorPrefix}Table .empty-message`;
            $(noResultsMessage).hide();
            $(`#${selectorPrefix}Table`).removeClass('table-empty');
            $(`#${selectorPrefix}Next`).addClass("disabled");
            $(`#${selectorPrefix}Previous`).addClass("disabled");
    
            //if dataTable has records, assume that it's the full list
            //if dataTable is empty, run the api call to return the records
            var dataTable = $(`#${selectorPrefix}Table-Full`).val();
    
            if (dataTable === "") {
                // Set up the AJAX request
                $.ajax({
                    url: serviceUrl,
                    type: 'GET',
                    contentType: 'application/json',
                    dataType: 'json',
                    headers: {"Access-Control-Allow-Origin": "*"},
                    success: function(response) {
                        var records = response;
                        var data = {
                            itemsPerPageCount: itemsPerPage,
                            totalRecordCount: records.length,
                            results: records
                        };
                        if (data.results.length > 0) {
                            $(loadingIconSelector).hide();
                            $('#submit-btn').prop('disabled', false);
                            $('#reset-btn').prop('disabled', false);
    
                            // Create a new array to hold the groups
                            const groupedPages = [];
    
                            // Loop through the dataset and create subarrays based on itemsPerPage
                            for (let i = 0; i < data.results.length; i += itemsPerPage) {
                                groupedPages.push(data.results.slice(i, i + itemsPerPage));
                            }
    
                            // Save the full data set of pages in a hidden control
                            $(`#${selectorPrefix}Table-Full`).val(JSON.stringify(groupedPages));
    
                            // Set up the first page
                            var dataPage = groupedPages[0]; 
                            _refreshTable(dataPage, selectorPrefix, createRowCallback);     
                            _refreshPageList(currentPage, data.itemsPerPageCount, data, setOnClickEvent, selectorPrefix);
                        } else {
                            $(loadingIconSelector).hide();
                            $('#submit-btn').prop('disabled', false);
                            $('#reset-btn').prop('disabled', false);
                            
                            _displayNoResults(selectorPrefix);
                            $( "#page-list" ).empty();
                            currentPage = previousPage;
                            $(`#${selectorPrefix}Next`).removeClass("disabled");
                            $(`#${selectorPrefix}Previous`).removeClass("disabled");
                        }
    
                        updateSubgridCount(selectorPrefix, data.results);
    
                        console.log('Response:', response);
                    },
                    error: function(xhr, status, error) {
                        console.error('Request failed. Status:', status, 'Error:', error);
                        currentPage = previousPage;
                    }
                });
            }
            else {
                var dataset = JSON.parse(dataTable);
                var flatDataSet = dataset.reduce((total, row) => total.concat(row), []);
                var data = {
                            itemsPerPageCount: itemsPerPage,
                            totalRecordCount: flatDataSet.length,
                            results: flatDataSet
                        };

                $('#submit-btn').prop('disabled', false);
                $('#reset-btn').prop('disabled', false);
    
                _refreshTable(dataset[currentPage-1], selectorPrefix, createRowCallback);
                _refreshPageList(currentPage, data.itemsPerPageCount, data, setOnClickEvent, selectorPrefix);
    
                updateSubgridCount(selectorPrefix, data.results);
    
                $(loadingIconSelector).hide();
            }
            
            return currentPage;
        };
		
		var handleApiRequest = function (serviceUrl, createRowCallback) {
			ND.Portal.API.appAjax('Retrieving records...', {
                type: "GET",
                url: serviceUrl,
                contentType: "application/json",
                success: function (response) {
                    var data = JSON.parse(response);
					for (let i = 0; i < data.results.length; i++) {
						createRowCallback(data.results[i]);
					}
                },
                failure: function (res) {
                }
            });
            
		}
        
        var _refreshTable = async function (results, selectorPrefix, createRowCallback, isSubgrid, refreshTableCallback) {
            //clear all rows
			$(`#${selectorPrefix}Table`).find("tbody>tr").not('.empty-message').remove();

            var callbackresults = [];
            //populate table with paginated data
            for (let i = 0; i < results.length; i++) {
                if (isSubgrid) {
                    callbackresults.push(createRowCallback(results[i], isSubgrid));
                } else {
                    callbackresults.push(createRowCallback(results[i]));
                }
            }

            var rows = await Promise.all(callbackresults);

            rows.forEach((rowHtml) => {
                $(`#${selectorPrefix}Table`).find("tbody").append(rowHtml);
            });

            // initialize any popover elements that were added
            $('[data-toggle="popover"]').popover();
			
			_handleEmptyValues(selectorPrefix);
            if (refreshTableCallback) {
                refreshTableCallback();
            }
        };

        var _refreshPageList = function (currentPage, itemsPerPage, result, setOnClickEvent, selectorPrefix, tableNumber) {
            var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);
            pageButtonList.empty();

            //making this reusable for components with pagination in a for loop
            var sectionSelector;
            if (result.thirdPartyId) {
                sectionSelector = result.thirdPartyId;
            }

            if (!itemsPerPage) itemsPerPage = 10;
            var totalPages = Math.ceil(result.totalRecordCount / itemsPerPage);

            //remove pagination for 1 page
            if (totalPages == 1) {
                return;
            }

            //add left arrow - disabled if on page 1
            var previousPage = currentPage - 1;
            if (previousPage == 0) {
                pageButtonList.append(
                    `<li class="disabled"><a id="${selectorPrefix}Previous" data-page="${previousPage}" aria-label="Previous page" title="Previous Page" class="entity-pager-prev-link" data-original-title="Previous page"><span aria-hidden="true">&lt;</span></a></li>`
                );
            } else {
                pageButtonList.append(
                    `<li><a href="javascript:void(0)" id="${selectorPrefix}Previous" data-page="${previousPage}" aria-label="Previous page" title="Previous Page" class="entity-pager-prev-link" data-original-title="Previous page"><span aria-hidden="true">&lt;</span></a></li>`
                );
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(previousPage, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(previousPage, tableNumber);
                        } else {
                            setOnClickEvent(previousPage);
                        }
                    });
            }

            //determine range of buttons around selected page

            //scenario 1: beginning of list
            if (currentPage < 8 || (currentPage == 8 && totalPages == 8)) {
                //if there are less than 8 pages don't need ellipses or last page
                if (totalPages <= 8) {
                    _createPageLinks(1, totalPages, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                } else {
                    _createPageLinks(1, 8, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                    //is this check really necessary??
                    if (totalPages != 9) {
                        pageButtonList.append(
                            `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                        );
                    }
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${totalPages}" data-page="${totalPages}" aria-current="false">${totalPages}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(totalPages, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(totalPages, tableNumber);
                            } else {
                                setOnClickEvent(totalPages);
                            }
                        });
                }
            }
            //display first page and ellipses for following two scenarios
            else {
                //display page 1
                pageButtonList.append(`<li><a href="javascript:void(0)" aria-label="page 1" data-page="1" aria-current="false">1</a></li>`);
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(1, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(1, tableNumber);
                        } else {
                            setOnClickEvent(1);
                        }
                    });
                //display ellipses
                pageButtonList.append(
                    `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                );

                //scenario 4: reached query limit but still additional pages
                if (currentPage > totalPages && result.moreRecords) {
                    totalPages = currentPage;
                }

                //scenario 2: showing end of list
                if (currentPage > totalPages - 7) {
                    _createPageLinks(totalPages - 7, totalPages, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                }
                //scenario 3: show middle of list
                else {
                    _createPageLinks(currentPage - 4, currentPage + 4, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                    //display ellipses
                    pageButtonList.append(
                        `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                    );
                    //display last page
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${totalPages}" data-page="${totalPages}" aria-current="false">${totalPages}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(totalPages, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(totalPages, tableNumber);
                            } else {
                                setOnClickEvent(totalPages);
                            }
                        });
                }
            }

            //add next arrow - disable if no more records
            var nextPage = currentPage + 1;
            if (nextPage <= totalPages || result.moreRecords) {
                //display extra pages if reached query limit but still additional pages
                if (currentPage >= totalPages && result.moreRecords) {
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${nextPage}" data-page="${nextPage}" aria-current="false">${nextPage}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(nextPage, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(nextPage, tableNumber);
                            } else {
                                setOnClickEvent(nextPage);
                            }
                        });
                }

                pageButtonList.append(
                    `<li><a href="javascript:void(0)" id="${selectorPrefix}Next" data-page="${nextPage}" aria-label="Next page" title="Next Page" class="entity-pager-next-link" data-original-title="Next page"><span aria-hidden="true">&gt;</span></a></li>`
                );
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(nextPage, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(nextPage, tableNumber);
                        } else {
                            setOnClickEvent(nextPage);
                        }
                    });
            } else {
                pageButtonList.append(
                    `<li class="disabled"><a id="${selectorPrefix}Next" data-page="${nextPage}" aria-label="Next page" title="Next Page" class="entity-pager-next-link" data-original-title="Next page"><span aria-hidden="true">&gt;</span></a></li>`
                );
            }
            if (currentPage != 1) {
                if ($(`#${selectorPrefix}-page-list-custom a[data-page="${currentPage}"]`).length !== 0) {
                    $(`#${selectorPrefix}-page-list-custom a[data-page="${currentPage}"]`).focus();
                    return;
                }

                // TO-DO: remove after all custom tables have been updated with _TableContainer
                $("a[data-page=" + currentPage + "]").focus();
            }
        };
        
        var _createPageLinks = function (startIndex, endIndex, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber) {
            var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);

            for (var i = startIndex; i <= endIndex; i++) {
                (function (buttonNum) {
                    pageButtonList.append(
                        `<li class="${
                            buttonNum == currentPage ? "active" : ""
                        }"><a href="javascript:void(0)" aria-label="page ${buttonNum}" data-page="${buttonNum}" aria-current="${
                            buttonNum == currentPage ? "page" : "false"
                        }">${buttonNum}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(buttonNum, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(buttonNum, tableNumber);
                            } else {
                                setOnClickEvent(buttonNum);
                            }
                        });
                })(i);
            }
        };
        
        var _showHidePageButtons = function (moreRecords, currentPage, selectorPrefix) {
            if (!moreRecords) {
                _disablePageButton(`#${selectorPrefix}Next`);
            } else {
                _enablePageButton(`#${selectorPrefix}Next`);
            }

            if (currentPage == 1) {
                _disablePageButton(`#${selectorPrefix}Previous`);
            } else {
                _enablePageButton(`#${selectorPrefix}Previous`);
            }

            if (!moreRecords && currentPage === 1) {
                _disablePageButton(`#${selectorPrefix}PageCount`);
            } else {
                _enablePageButton(`#${selectorPrefix}PageCount`);
            }
        };

        var _enablePageButton = function (buttonSelector) {
            $(buttonSelector).parent().show();
        };

        var _disablePageButton = function (buttonSelector) {
            $(buttonSelector).parent().hide();
        };

		var _handleEmptyValues = function(tableName) {
			$(`#${tableName}Table td a:empty`).replaceWith('&ndash;');
		}
        
        var _displayNoResults = function (selectorPrefix) {
            $(`#${selectorPrefix}Table`).addClass('table-empty');
            $(`#${selectorPrefix}Table`).find("tbody>tr").not('.empty-message').remove();
            $(`#${selectorPrefix}Table .empty-message`).show();
        };

        const handleTableSorting = function (target, tableSettingsConfiguration) {
            const $parentTable = $(target).parents("table");
            const tableIdentifier = $parentTable.attr("data-ordering");
            if (!tableIdentifier) {
                return;
            }

            const tableSettings = tableSettingsConfiguration[tableIdentifier];

            tableSettings.OrderingConfiguration = _updateOrderingConfiguration(target, tableSettings?.InvertedSorting);
            tableSettings.PagingCallback(1);
        };
        
        const _updateOrderingConfiguration = function (element, invertedSorting = []) {
            const $element = $(element);

            const orderOn = $element.attr("data-attribute");
            if (!orderOn) {
                return;
            }
            let inverted = invertedSorting.indexOf(orderOn) !== -1;
            let className = inverted ? "sort-desc" : "sort-asc";

            let orderBy;
            if ($element.parent().hasClass(className)) {
                $element.parent().removeClass(className);
                orderBy = inverted ? "ascending" : "descending";
            } else {
                $element.parent().addClass(className);
                orderBy = inverted ? "descending" : "ascending";
            }
            
            return {
                OrderOn: orderOn,
                OrderBy: orderBy,
                Inverted: inverted
            };
        };

        const clearTableSorting = function ($tableColumn) {
            $tableColumn.parents("tr").children("th").removeClass("sort sort-asc sort-desc");
            $tableColumn.parents("tr").children("th").removeAttr("aria-sort");
            $tableColumn.parents("tr").find("span.fa").remove();
            $tableColumn.parents("tr").find("span.sort-hint").remove();
        };

        const resetTableSorting = function (tableIdentifier, orderingConfiguration) {
            $(`#${tableIdentifier}TableContainer`).css("min-height", "0px");

            if (!orderingConfiguration?.OrderOn) {
                return;
            }

            const $a = $(`#${tableIdentifier}Table th.sort-enabled a[data-attribute="${orderingConfiguration.OrderOn}"]`);

            clearTableSorting($a);

            const orderBy = orderingConfiguration?.OrderBy || "ascending";
            const orderByClassName = !orderBy ? "sort-asc" : orderBy === "ascending" ? "sort-asc" : "sort-desc";
            $a.parent().addClass(`sort ${orderByClassName}`);
            $a.parent().attr("aria-sort", orderBy);

            let sortingIconClassName = orderByClassName === "sort-asc" ? `fa-arrow-${orderingConfiguration?.Inverted ? 'up' : 'down'}` : `fa-arrow-${orderingConfiguration?.Inverted ? 'down' : 'up'}`;
            
            $a.append(`<span class="fa ${sortingIconClassName}" aria-hidden="true"></span>`);
            $a.append(`<span class="sr-only sort-hint">. sort ${orderBy}</span>`);
        };
        
        var resetTablePaging = function (selectorPrefix) {
            $('.pagination li').removeClass('active');
            $('.pagination li>a[data-page="1"]').parent().addClass('active');
        }

        var renderSubgrid = (htmlId, sectionLabel) => {
			var html = $(`#${htmlId}`);
            var selector = `fieldset[aria-label="${sectionLabel}"]`;
            $(selector).children().remove()
			$(selector).append(html.children());
        }

        var prependToSection = (sectionName, fieldsetToAdd) => {
            $(`fieldset[aria-label="${sectionName}"]`).prepend($(`fieldset[aria-label="${fieldsetToAdd}"]`).children());
        }

        var updateSubgridCount = (id, results, count) => {
            count = count ?? (results.length > 0 ? results[0].TotalRecordCount : 0)
            if (results && results.length > 0 && count > 0) {
                $(`#${id}-count`).show();
                $(`#${id}-count`).removeClass("hidden");
                $(`#${id}-count-value`).text(count);
            }
            else {
                $(`#${id}-count`).hide();
            }
        }

        var filterBarHasValues = (selector) => {
            let hasValues = false;
            $(`${selector} .form-layout select, ${selector} .form-layout input`).not("#openingBidPerAcre-input, #update-request-modal select, #update-request-modal input").each(function () {
                if ($(this).val()) {
                    hasValues = true;
                }
            });
            return hasValues;
        }
        
        return {
            handlePageToggle: handlePageToggle,
            handleSubgridPageToggle: handleSubgridPageToggle,
            handleSubgridTablePaging: handleSubgridTablePaging,
            writeAdditionalValuesToPortalData: writeAdditionalValuesToPortalData,
            handleTablePaging: handleTablePaging,
            handleTableSorting: handleTableSorting,
            clearTableSorting: clearTableSorting,
            resetTableSorting: resetTableSorting,
            resetTablePaging: resetTablePaging,
            renderSubgrid: renderSubgrid,
            prependToSection: prependToSection,
            updateSubgridCount: updateSubgridCount,
            handleApiTable: handleApiTable,
            filterBarHasValues: filterBarHasValues
        }
    })();

})(this);